-- This file is a part of Avoision.
-- 
-- Copyright (C) 2010 Jared Krinke.
-- 
-- Avoision is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

-- TODO: This is a layer and it would make sense to group layers by name or in a folder
require("UI/UI.lua")
require("Avoision/Common.lua")
require("Avoision/CreditsMenu.lua")
require("Avoision/GameLayer.lua")
require("Avoision/HighScore.lua")
require("Avoision/HighScoreEntryMenu.lua")
require("Avoision/HighScoreMenu.lua")
require("Avoision/OptionsMenu.lua")
require("Avoision/ScriptedEntity.lua")
require("Utility/MiscSettings.lua")

Logo = { }

function Logo.new()
    local logo = Entity.new()

    logo.desiredWidth = 1
    logo.desiredHeight = 1

    -- Animation
    local chaseX = 0.2
    local chaseY = -0.2
    local chaseX2 = 0.8
    local chaseY2 = -0.8
    local chaseSize = 0.25
    local chasePeriod = 3000
    
    logo.children = EntityList.new()
    logo.children:add(Entity.new(0.5, -0.5, -1, 0.95, 0.95, 0, Color.new(1, 1, 1), ElementList.new(Element.Image.new("Images/Background.png"))))
    logo.children:add(ScriptedEntity.new({ Element.Image.new("Images/Block.png") },
                                             { { 0,               chaseX,  chaseY,  0, chaseSize, chaseSize, 0, Color.new(0, 1, 0) },
                                               { chasePeriod / 4, chaseX2, chaseY,  0, chaseSize, chaseSize, 0, Color.new(0, 1, 0) },
                                               { chasePeriod / 4, chaseX2, chaseY2, 0, chaseSize, chaseSize, 0, Color.new(0, 1, 0) },
                                               { chasePeriod / 4, chaseX,  chaseY2, 0, chaseSize, chaseSize, 0, Color.new(0, 1, 0) },
                                               { chasePeriod / 4, chaseX,  chaseY, 0, chaseSize, chaseSize, 0, Color.new(0, 1, 0) } },
                                             true))
    
    logo.children:add(ScriptedEntity.new({ Element.Image.new("Images/Block.png") },
                                             { { 0,               chaseX2, chaseY,  0, chaseSize, chaseSize, 0, Color.new(1, 0, 0) },
                                               { chasePeriod / 4, chaseX2, chaseY2, 0, chaseSize, chaseSize, 0, Color.new(1, 0, 0) },
                                               { chasePeriod / 4, chaseX,  chaseY2, 0, chaseSize, chaseSize, 0, Color.new(1, 0, 0) },
                                               { chasePeriod / 4, chaseX,  chaseY,  0, chaseSize, chaseSize, 0, Color.new(1, 0, 0) },
                                               { chasePeriod / 4, chaseX2, chaseY,  0, chaseSize, chaseSize, 0, Color.new(1, 0, 0) } },
                                             true))
    
    logo.children:add(ScriptedEntity.new({ Element.Image.new("Images/Block.png") },
                                             { { 0,               chaseX,  chaseY2, 0, chaseSize, chaseSize, 0, Color.new(1, 1, 1) },
                                               { chasePeriod / 4, chaseX,  chaseY,  0, chaseSize, chaseSize, 0, Color.new(1, 1, 1) },
                                               { chasePeriod / 4, chaseX2, chaseY,  0, chaseSize, chaseSize, 0, Color.new(1, 1, 1) },
                                               { chasePeriod / 4, chaseX2, chaseY2, 0, chaseSize, chaseSize, 0, Color.new(1, 1, 1) },
                                               { chasePeriod / 4, chaseX,  chaseY2, 0, chaseSize, chaseSize, 0, Color.new(1, 1, 1) } },
                                             true))

    -- Component implementation
    function logo.setLayer(self, layer)
        layer.entities:add(self)
    end

    function logo.getPosition(self)
        return self.x, self.y
    end

    function logo.setPosition(self, x, y)
        self.x = x
        self.y = y
    end

    function logo.getActive(self)
        return false
    end

    function logo.getMinimumSize(self)
        return self.desiredWidth, self.desiredHeight
    end

    logo.getDesiredSize = logo.getMinimumSize
    logo.getSize = logo.getMinimumSize

    function logo.setSize(self, width, height)
        -- Maintain aspect ratio
        self.desiredWidth = max(width, height)
        self.desiredHeight = self.desiredWidth

        self.width = self.desiredWidth
        self.height = self.desiredWidth
    end

    return logo
end

MainMenu = { }

function MainMenu.new()
    local mainForm = Form.newGrid(1, -200, 200, 400)

    mainForm.gameLayer = GameLayer.new(function(difficulty, score)
        if score > HighScore.getLowestScore(difficulty) then
            mainForm:showHighScoreEntry(difficulty, score, function() mainForm:showHighScores() end)
        else
            mainForm:showHighScores()
        end
    end)

    mainForm.creditsMenu = CreditsMenu.new()
    mainForm.highScoreEntryMenu = HighScoreEntryMenu.new()
    mainForm.highScoreMenu = HighScoreMenu.new()
    mainForm.optionsMenu = OptionsMenu.new()

    function mainForm.startNewGame(self)
        self.gameLayer:setDifficulty(self.difficulty)
        self.gameLayer:reset()
        Layers.push(self.gameLayer)
        self.gameLayer:started()
    end

    function mainForm.showHighScoreEntry(self, difficulty, score, entered)
        self.highScoreEntryMenu:show(difficulty, score, entered)
    end

    function mainForm.showHighScores(self)
        self.highScoreMenu:show()
    end

    function mainForm.showVideoOptions(self)
        self.videoMenu:show()
    end
    
    function mainForm.exit(self)
        exit()
    end

    -- Form
    local difficultyChoice = Choice.new("Difficulty", Avoision.difficultyLevelNames, function(self, choice) mainForm.difficulty = Avoision.difficultyLevel[choice] end)

    mainForm:add(Form.newNestedCenterFlow(2, Title.new("Avoision ", 1.75), Logo.new()))
    mainForm:add(Form.newNestedLeftFlow(1,
        Separator.new(),
        Button.new("New Game", function()
            -- Save difficulty selection
            MiscSettings.set("string", "difficulty", Avoision.difficultyLevelNames[mainForm.difficulty])
            MiscSettings.save()

            -- Start game
            mainForm:startNewGame()
        end, true),

        difficultyChoice,
        Button.new("High Scores", function() mainForm:showHighScores() end),
        Button.new("Options", function() mainForm.optionsMenu:show()  end),
        Button.new("Credits", function() mainForm.creditsMenu:show() end, true),
        Separator.new(),
        Button.new("Quit", function() mainForm:exit() end)))

    -- Default to initial difficulty
    local initialDifficulty = MiscSettings.get("string", "difficulty")

    if isString(initialDifficulty) then
        difficultyChoice:select(initialDifficulty)
    end

    -- Center form vertically
    local x, y = mainForm:getPosition()
    local width, height = mainForm:getMinimumSize()

    mainForm:setPosition(x, height / 2)

    -- Create layer
    local mainMenu = FormLayer.new(mainForm, 33)

    -- Version
    local version = Entity.new(-315, -235, 0, Avoision.textWidth / 2, Avoision.textHeight / 2)

    version.elements:add(Element.Text.new("v" .. Avoision.version))
    version.color = Color.new(0.5, 0.5, 0.5)
    mainMenu.entities:add(version)
    
    -- Error handling
    function mainMenu.errorOccurred(self, message)
        print(message)
        console:show()
    end

    -- Console for debugging (TODO: remove eventually or only add to debug versions)
    -- TODO: Re-enable console
    function mainMenu.inputReceived(self, event, pressed)
        local handled = false

        if event == "openConsole" and pressed then
            console:show()
            handled = true
        else
            handled = mainForm:inputReceived(event, pressed)
        end

        return handled
    end

    return mainMenu
end

