-- This file is a part of Avoision.
-- 
-- Copyright (C) 2010 Jared Krinke.
-- 
-- Avoision is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

local cursorWidth = 0.3
local cursorSpace = 0.2

TextInputBox = { }

function TextInputBox.new(submitted, initialText, historySize)
    local textInputBox = Entity.new()

    textInputBox.submitted = submitted
    textInputBox.buffer = StringBuffer.new()
    textInputBox.textElement = Element.Text.new()
    textInputBox.cursor = Element.Image.new("Images/Blank.png", 0, 0.5, -1, cursorWidth, 1, 0, Color.new(1, 1, 1, 0.3))
    textInputBox.cursorPosition = 1

    function textInputBox.clear(self)
        self:setCursorPosition(1)
        self.buffer:clear()
    end
    
    function textInputBox.setCursorPosition(self, position)
        self.cursorPosition = position
        self.cursor.x = position - (1 - cursorSpace)
    end
    
    function textInputBox.append(self, character)
        self.buffer:insert(character, self.cursorPosition)
        self:setCursorPosition(self.cursorPosition + 1)
    end
    
    function textInputBox.appendSpace(self, character)
        self:append(' ')
    end
    
    function textInputBox.backspace(self)
        if self.cursorPosition > 1 then
            self.buffer:remove(self.cursorPosition - 1, 1)
            self:setCursorPosition(self.cursorPosition - 1)
        end
    end
    
    function textInputBox.delete(self)
        if self.cursorPosition <= self.buffer.length then
            self.buffer:remove(self.cursorPosition, 1)
        end
    end
    
    function textInputBox.moveLeft(self)
        if self.cursorPosition > 1 then
            self:setCursorPosition(self.cursorPosition - 1)
        end
    end
    
    function textInputBox.moveRight(self)
        if self.cursorPosition <= self.buffer.length then
            self:setCursorPosition(self.cursorPosition + 1)
        end
    end
    
    function textInputBox.enter(self)
        if isFunction(self.submitted) then
            self:submitted()
        end

        Audio.play("Sounds/Select.wav")
    end
    
    function textInputBox.keyPressed(self, key, pressed, character)
        if pressed then
            local handler = self.handlers[key]
    
            if isFunction(handler) then
                handler(self, character)
            end
        end
    end

    function textInputBox.getText(self)
        return self.buffer.text
    end

    -- Component methods
    function textInputBox.setLayer(self, layer)
        layer.entities:add(self)
    end

    function textInputBox.getPosition(self)
        return self.x, self.y + self.height
    end

    function textInputBox.setPosition(self, x, y)
        self.x = x
        self.y = y - self.height
    end

    function textInputBox.setTextSize(self, width, height)
        self.width = width
        self.height = height
    end

    function textInputBox.getMinimumSize(self)
        -- Leave some minimum amount of space
        local length = max(#self.buffer.text + 1, 10)

        return length * self.width, self.height
    end

    textInputBox.getDesiredSize = textInputBox.getMinimumSize
    textInputBox.getSize = textInputBox.getMinimumSize

    function textInputBox.setSize(self, width, height)
        -- Ignore size changes other than text size
    end

    function textInputBox.focused(self)
        self.cursor.color.opacity = 1
    end

    function textInputBox.unfocused(self)
        self.cursor.color.opacity = 0
    end

    -- TODO: Implement command history
    if isNumber(historySize) and historySize > 0 then
        textInputBox.historySize = historySize

        textInputBox.history = RingBuffer.new(textInputBox.historySize)
    else
        textInputBox.historySize = 0
    end

    textInputBox.textElement.buffer = textInputBox.buffer
    
    textInputBox.elements:add(textInputBox.textElement)
    textInputBox.elements:add(textInputBox.cursor)

    textInputBox.handlers = {
        ['a'] = textInputBox.append,
        ['b'] = textInputBox.append,
        ['c'] = textInputBox.append,
        ['d'] = textInputBox.append,
        ['e'] = textInputBox.append,
        ['f'] = textInputBox.append,
        ['g'] = textInputBox.append,
        ['h'] = textInputBox.append,
        ['i'] = textInputBox.append,
        ['j'] = textInputBox.append,
        ['k'] = textInputBox.append,
        ['l'] = textInputBox.append,
        ['m'] = textInputBox.append,
        ['n'] = textInputBox.append,
        ['o'] = textInputBox.append,
        ['p'] = textInputBox.append,
        ['q'] = textInputBox.append,
        ['r'] = textInputBox.append,
        ['s'] = textInputBox.append,
        ['t'] = textInputBox.append,
        ['u'] = textInputBox.append,
        ['v'] = textInputBox.append,
        ['w'] = textInputBox.append,
        ['x'] = textInputBox.append,
        ['y'] = textInputBox.append,
        ['z'] = textInputBox.append,
        ['0'] = textInputBox.append,
        ['1'] = textInputBox.append,
        ['2'] = textInputBox.append,
        ['3'] = textInputBox.append,
        ['4'] = textInputBox.append,
        ['5'] = textInputBox.append,
        ['6'] = textInputBox.append,
        ['7'] = textInputBox.append,
        ['8'] = textInputBox.append,
        ['9'] = textInputBox.append,
        ['-'] = textInputBox.append,
        ['='] = textInputBox.append,
        ['['] = textInputBox.append,
        [']'] = textInputBox.append,
        ['\\'] = textInputBox.append,
        [';'] = textInputBox.append,
        ['\''] = textInputBox.append,
        [','] = textInputBox.append,
        ['.'] = textInputBox.append,
        ['/'] = textInputBox.append,
        ['`'] = textInputBox.append,
        ["space"] = textInputBox.appendSpace,
        ["backspace"] = textInputBox.backspace,
        ["delete"] = textInputBox.delete,
        ["left"] = textInputBox.moveLeft,
        ["right"] = textInputBox.moveRight,
        ["enter"] = textInputBox.enter,
        ["escape"] = textInputBox.clear,
    }


    -- Set default text and cursor position
    if isString(initialText) then
        textInputBox.buffer:append(initialText)
        textInputBox:setCursorPosition(#initialText + 1)
    else
        textInputBox:setCursorPosition(1)
    end

    return textInputBox
end

